<?php

function neo4j_admin_form($form) {
  $connection = _neo4j_get_connection();
  $parts = parse_url($connection);

  $form['connection'] = [
    '#type' => 'fieldset',
    '#title' => t('Connection'),
  ];

  $form['connection']['protocol'] = [
    '#title' => t('Protocol'),
    '#type' => 'radios',
    '#default_value' => empty($parts['scheme']) ? 'bolt' : $parts['scheme'],
    '#options' => drupal_map_assoc(['bolt', 'http']),
    '#required' => TRUE,
  ];

  $form['connection']['user'] = [
    '#title' => t('Username'),
    '#type' => 'textfield',
    '#default_value' => empty($parts['user']) ? '' : $parts['user'],
    '#required' => TRUE,
  ];

  $form['connection']['pass'] = [
    '#title' => t('Password'),
    '#type' => 'password',
    '#default_value' => empty($parts['pass']) ? '' : $parts['pass'],
    '#required' => TRUE,
  ];

  $form['connection']['host'] = [
    '#title' => t('Host'),
    '#type' => 'textfield',
    '#default_value' => empty($parts['host']) ? '' : $parts['host'],
    '#required' => TRUE,
  ];

  $form['connection']['port'] = [
    '#title' => t('Port'),
    '#type' => 'textfield',
    '#default_value' => empty($parts['port']) ? '' : $parts['port'],
  ];

  $form['submit'] = [
    '#type' => 'submit',
    '#value' => t('Save'),
  ];

  return $form;
}

function neo4j_admin_form_validate($form, &$form_state) {
  static $defaultPort = [
    'bolt' => 7687,
    'http' => 7474,
  ];

  $protocol = $form_state['values']['protocol'];
  $user = $form_state['values']['user'];
  $pass = $form_state['values']['pass'];
  $host = $form_state['values']['host'];
  $port = $form_state['values']['port'];

  if (!$port) {
    $port = $defaultPort[$protocol];
  }

  $connection = "{$protocol}://{$user}:{$pass}@{$host}:{$port}";

  $result = NULL;
  $errorMessage = NULL;
  try {
    $client = \GraphAware\Neo4j\Client\ClientBuilder::create()
      ->addConnection('default', $connection)
      ->build();

    $result = $client->run('MATCH (n) RETURN n LIMIT 0');
  } catch (Exception $e) {
    $errorMessage = $e->getMessage();
  }
  if (!$errorMessage) {
    $errorMessage = t("Failed to connect");
  }

  if ($result) {
    $form_state['values']['connection'] = $connection;
  }
  else {
    form_set_error('', $errorMessage);
  }
}

function neo4j_admin_form_submit($form, &$form_state) {
  $connection = $form_state['values']['connection'];
  variable_set('neo4j_connection', $connection);
}

function neo4j_admin_log_form($form) {
  $form['neo4j_page_log'] = [
    '#type' => 'checkbox',
    '#title' => t('Page log'),
    '#default_value' => variable_get('neo4j_page_log'),
  ];

  return system_settings_form($form);
}
